#include "StdAfx.h"
#include "DrawAndMouseHandler.h"
#include "TutorialApp.h"

CDrawOnlyHandler::CDrawOnlyHandler()
{
}

bool CDrawOnlyHandler::Initialize(CDialog& parent_dlg, int control_id)
{
  if( !m_window.ReplaceDialogControl(parent_dlg.m_hWnd, control_id) )
    return false;

  if( !m_window.AttachHandler(this) )
    return false;

  return true;
}

void CDrawOnlyHandler::DrawGraphics()
{
  float width, height;
  theApp.GetModel().GetWorldSize(width, height);
  width = height * GetAspectRatio();

  if(NULL == m_pD3DDevice || width <= 0 || height <= 0)
    return;

  BeginDraw();
  
  // Construct the matrix that will transform from the world bound to the Viewport
  D3DXMATRIX world2ndc;
  D3DXVECTOR3 scale(2.0f/width, 2.0f/height, 1.0f);
  D3DXVECTOR3 translate(-1.0f, -1.0f, 0.0);
  D3DXMatrixTransformation(&world2ndc, 
                            NULL,   // Scale reference
                            NULL,   // Scale rotation
                            &scale, // the actual Scales
                            NULL,   // Axis of rotation
                            NULL,   // The rotation
                            &translate);
  m_pD3DDevice->SetTransform(D3DTS_VIEW, &world2ndc);

  //clear the draw buffer to light gray
  D3DCOLOR background = D3DCOLOR_COLORVALUE(0.8f, 0.8f, 0.8f, 0);
  m_pD3DDevice->Clear(0,                // number of rectangles, 0 means the entire area
                      NULL,             // rectangles to be cleared, NULL means the entire area
                      D3DCLEAR_TARGET,  // clear target (color) (we are not changing the z buffer)
                      background,// new color to clear to
                      0.0f,             // new Z values to clear to
                      0);               // new Stencil buffer values to clear to

  //perform all of our drawing
  theApp.GetModel().DrawModel();

  EndDrawAndShow();
}

bool CDrawOnlyHandler::GetWindowClientRect(CRect& rect) const
{
  if( !IsWindow(m_window) )
    return false;

  m_window.GetClientRect(rect);
  return true;
}

//--------------------------------------------------------------------------------------------
CDrawAndMouseHandler::CDrawAndMouseHandler()
{
}

void CDrawAndMouseHandler::OnMouseButton(bool down, unsigned int nFlags, int hwX, int hwY)
{
  int deviceX, deviceY;
  HardwareToDevice(hwX, hwY, deviceX, deviceY);
  CString msg;

  if(nFlags & MK_LBUTTON)
  {
    if(down)
    {
      vec3 p(float(deviceX), float(deviceY), 0.0f);
      theApp.GetModel().SetGuidePosition(p);  
      theApp.GetModel().m_Ball.SetVelocity(vec3(0));
      msg.Format(L"LMB Down:(%d %d)", deviceX, deviceY);
	    UWB_EchoToStatusArea(msg);
    }
    else 
    {
      vec3 pos(float(deviceX), float(deviceY), 0.0f);
      float len = length(theApp.GetModel().m_Guide.GetCenter() - pos);
      vec3 dir  = theApp.GetModel().m_Guide.GetCenter() - theApp.GetModel().m_Ball.GetCenter();
      normalize(dir);
      dir *= len;
      theApp.GetModel().SetHeroBallVelocity(dir);
      msg.Format(L"LMB UP:(%d %d). Final Speed(%6.2f)", deviceX, deviceY, len);
	    UWB_EchoToStatusArea(msg);
    }
  }

  if(nFlags & MK_RBUTTON)
  {
    if(down)
    {
      vec3 p(float(deviceX), float(deviceY), 0.0f);
      theApp.GetModel().SetHeroBallPosition(p);
      msg.Format(L"RMB Down:(%d %d)", deviceX, deviceY);
	    UWB_EchoToStatusArea(msg);
    }
  }
}

void CDrawAndMouseHandler::OnMouseMove(unsigned int nFlags, int hwX, int hwY)
{
  CString msg;
  int deviceX, deviceY;
  HardwareToDevice(hwX, hwY, deviceX, deviceY);


  if(nFlags & MK_LBUTTON)
  {
    vec3 pos(float(deviceX), float(deviceY), 0.0f);
    float len = length(theApp.GetModel().m_Guide.GetCenter() - pos);
    msg.Format(L"LMB Drag:(%d %d). Current Speed(%6.2f)", deviceX, deviceY, len);
	  UWB_EchoToStatusArea(msg);
  }

  if(nFlags & MK_RBUTTON)
  {
    vec3 pos(float(deviceX), float(deviceY), 0.0f);
    theApp.GetModel().SetHeroBallPosition(pos);
    msg.Format(L"RMB Drag:(%d %d).", deviceX, deviceY);
	  UWB_EchoToStatusArea(msg);
  }
}
