// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once

/// Class with specific duty of drawing Direct3D graphics to an attached window
/// This implementation contains a member that represents that actual window
class CDrawOnlyHandler :  public UWBD3D_WindowHandler
{
public:
  CDrawOnlyHandler();

  /// Used to place the graphics window on a parent dialog
  /// \param parent_dlg The dialog to place the window on
  /// \param control_id ID for placeholder on dialog to replace with the graphics window
  /// \return true on success
  bool Initialize(CDialog& parent_dlg, int control_id);

  ///
  /// virtual function for drawing the actual graphics
  void DrawGraphics();

  /// Get the client rectangle of the attached window
  /// \param rect [out] on success, this is set to the client rectangle of the attached window
  /// \return true on success
  bool GetWindowClientRect(CRect& rect) const;

protected:
  ///
  /// The window that this handler draws graphics for
  UWBMFC_UIWindow m_window;
};

/// Class with specific duty of drawing Direct3D graphics to an attached window
/// and responding to mouse input on that window.
/// This implementation extends the draw only handler by adding mouse event handlers
class CDrawAndMouseHandler : public CDrawOnlyHandler 
{
public:
  CDrawAndMouseHandler();

  ///
  /// Called when mouse button messages are sent to the attached window
  void OnMouseButton(bool down, unsigned int nFlags, int windowX, int windowY);

  ///
  /// Called when mouse move messages are sent to the attached window
  void OnMouseMove(unsigned int nFlags, int windowX, int windowY);

private:
};
