// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include <time.h> // for clock_t data type
#include "Ball.h"

/// The model separates the geometry and simulation of geometry interactions
/// from the user interface. Separation of the model from the user interface
/// generally leads to easier to maintain applications

#define kBallSize 25

#define VERY_LARGE  9999999.0f
#define VERY_SLOW   0.001f
#define RADIUS_TOL  1.2f


class CModel
{
public:
  CModel();

  /// This model defines an area that is considered the "world" size. Anything outside that area
  /// is treated as not existing.
  /// Set the area that is considered the world
  /// \param width Width of the world area ( >=0)
  /// \param height Height of the world area ( >=0 )
  void SetWorldSize(float width, float height);
  void GetWorldSize(float& width, float& height) const;

  // Hero ball parameter
  void SetHeroBallVelocity(const vec3& v);
  void SetHeroBallPosition(const vec3& p);
  void SetGuidePosition(const vec3& p);

 
  /// Update the state of the model. This is used to update the objects contained in
  /// the model after a certain amount of time has passed. 
  void UpdateSimulation();

  ///
  /// Draw all of the geometry that is part of this model.
  void DrawModel();


  CBall  m_Ball;
  CBall  m_Guide;

public:
  void UpdateABall(CBall&, float);

  /// size of the world
  float m_WorldWidth, m_WorldHeight;

  // to slow down ...
  bool m_ShouldSlowDown;

  // to keep track of wall-clock elapses in between updates
  UWB_Clock m_stop_watch;
};