#include "uwbgl_color1.h"
#include "uwbgl_utility2.h"

UWB_Color::UWB_Color()
: m_color(0)
{
}

UWB_Color::UWB_Color(int red, int green, int blue, int alpha)
{
  SetRGBA(red, green, blue, alpha);
}

UWB_Color::UWB_Color(double percent_red, double percent_green, double percent_blue, double percent_alpha)
{
  int red = UWB_Round(percent_red * 255.0);
  int green = UWB_Round(percent_green * 255.0);
  int blue = UWB_Round(percent_blue * 255.0);
  int alpha = UWB_Round(percent_alpha * 255.0);
  SetRGBA(red, green, blue, alpha);
}

UWB_Color::UWB_Color(float percent_red, float percent_green, float percent_blue, float percent_alpha)
{
  int red = UWB_Round(percent_red * 255.0);
  int green = UWB_Round(percent_green * 255.0);
  int blue = UWB_Round(percent_blue * 255.0);
  int alpha = UWB_Round(percent_alpha * 255.0);
  SetRGBA(red, green, blue, alpha);
}

UWB_Color::UWB_Color(COLORREF rgb, int alpha)
{
  int red = GetRValue(rgb);
  int green = GetGValue(rgb);
  int blue = GetBValue(rgb);
  SetRGBA(red, green, blue, alpha);
}

bool UWB_Color::operator ==(const UWB_Color& other) const
{
  return m_color == other.m_color;
}

bool UWB_Color::operator !=(const UWB_Color& other) const
{
  return m_color != other.m_color;
}
void UWB_Color::SetRGBA(int red, int green, int blue, int alpha)
{
  if (red < 0)
    red = 0;
  else if ( red > 255 )
    red = 255;
  
  if (green < 0)
    green = 0;
  else if ( green > 255 )
    green = 255;
  
  if (blue  < 0 )
    blue  = 0;
  else if ( blue  > 255 )
    blue  = 255;

  if (alpha < 0 )
    alpha = 0;
  else if ( alpha > 255 )
    alpha = 255;

  //pack individual color channels into a single unsigned int
  m_color = (alpha << 24 ) | (red << 16) | (green << 8) | blue;
}

unsigned int UWB_Color::ARGB() const
{
  return m_color;
}

int UWB_Color::Red() const
{
  return (m_color>>16) & 0xFF;
}

int UWB_Color::Green() const
{
  return (m_color>>8) & 0xFF;
}

int UWB_Color::Blue() const
{
  return (m_color) & 0xFF;
}

int UWB_Color::Alpha() const
{
  return (m_color>>24) & 0xFF;
}

float UWB_Color::PercentRed() const
{
  return ( (float)Red() / 255.0f );
}

float UWB_Color::PercentGreen() const
{
  return ( (float)Green() / 255.0f );
}

float UWB_Color::PercentBlue() const
{
  return ( (float)Blue() / 255.0f );
}

float UWB_Color::PercentAlpha() const
{
  return ( (float)Alpha() / 255.0f );
}