// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////

#pragma once
#include "uwbgl_Common.h"

/// Color definition class. Diferent graphics API technologies like to use colors
/// in different ways. Most pack the color into a single unsigned integer, but the
/// way it is packed can vary between APIs. Some packings are ARGB, RGBA, and ABGR. This
/// class is used to hold a color in a packing of our choice and provides functions to
/// retrieve colors in different packing styles.
class UWB_Color
{
public:
  ///
  ///default constructor creates black (m_color == 0x00000000)
  UWB_Color();

  /// \param red, green, blue, alpha [in]: color values for each color channel. Should be
  ///                                      between 0-255
  UWB_Color(int red, int green, int blue, int alpha = 255);

  /// \param percent_red, percent_green, percent_blue, percent_alpha [in]:
  ///        percentage levels for each color channel. Should be between 0.0 - 1.0
  UWB_Color(double percent_red, double percent_green, double percent_blue, double percent_alpha = 1.0);

  /// \param percent_red, percent_green, percent_blue, percent_alpha [in]:
  ///        percentage levels for each color channel. Should be between 0.0 - 1.0
  UWB_Color(float percent_red, float percent_green, float percent_blue, float percent_alpha = 1.0f);

  /// \param rgb [in]: COLORREF is a common Win32 packing of red, green, and blue into a single value
  /// \param alpha [in]: COLORREF does not have an alpha value packed into it.
  UWB_Color(COLORREF rgb, int alpha = 255);

  ///
  /// Equality operator. Just comapares m_color values
  bool operator==(const UWB_Color& other) const;
  bool operator!=(const UWB_Color& other) const;

  /// Set each color channel for the class
  /// \param red, green, blue, alpha [in]: color values between 0-255
  void SetRGBA(int red, int green, int blue, int alpha);

  // return color packed as alpha,red,green,blue
  // This is the same packing as a D3DCOLOR
  unsigned int ARGB() const;

  ///
  /// return int value for red channel (0-255)
  int Red() const;
  ///
  /// return int value for green channel (0-255)
  int Green() const;
  ///
  /// return int value for blue channel (0-255)
  int Blue() const;
  ///
  /// return int value for alpha channel (0-255)
  int Alpha() const;

  ///
  /// return % of red color channel (0.0-1.0)
  float PercentRed() const;
  ///
  /// return % of green color channel (0.0-1.0)
  float PercentGreen() const;
  ///
  /// return % of blue color channel (0.0-1.0)
  float PercentBlue() const;
  ///
  /// return % of alpha color channel (0.0-1.0)
  float PercentAlpha() const;

private:
  /// Color is stored as a single unsigned int with the four color channels packed into the int
  /// int packing is as follows: 0xaarrggbb
  unsigned int m_color;
};

///
/// Definition for "undefined" color. Useful for testing a given color against
#define UWB_NOCOLOR 0xFFFFFFFF
