// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////

/// uwbgl_common.h: Common preprocessor symbols and macros used by the Graphics Library.
/// No version number was added to this header file because it doesn't change as we add
/// new functionality to the graphics libraries
#pragma once

#define UWB_PI 3.14159265358979323846f

#define UWB_ToRadians( degree ) ((degree) * (UWB_PI / 180.0f))
#define UWB_ToDegrees( radian ) ((radian) * (180.0f / UWB_PI))


#define UWB_VERY_LARGE 999999999.0f
#define UWB_VERY_SMALL -UWB_VERY_LARGE
#define UWB_ZERO_TOLERANCE 1.0e-9


#define UWBGL_EXTERNC extern "C"
#define UWBGL_BEGIN_EXTERNC extern "C" {
#define UWBGL_END_EXTERNC   }

// If the library is compiled as a DLL, we need to declare which classes
// and functions are publically exposed. The following ifdef block is the
// standard way of creating macros which make exporting from a DLL simpler.
// If we are compiling this project as a DLL, UWBGL_DLL_EXPORTS should be
// defined as a project preprocessor symbol. Any project that uses this DLL
// would have to define UWBGL_DLL_IMPORTS as a preprocessor symbol.
// If we are compiling the project as a static library, the UWBGL_API preprocessor
// symbol should resolve to nothing.
// NOTE: This library is written to work as a static library for simplicity. If
//       you wanted to make this a DLL, you will have to add UWBGL_API to every class
//       and function declaration. (i.e class UWBGL_API UWB_Color{...};

#if defined(UWBGL_DLL_EXPORTS)
  // When compiling as a DLL, export classes, functions, templates, and globals
  #define UWBGL_API __declspec(dllexport)
#elif defined(UWBGL_DLL_IMPORTS)
  // When an application is using this library as a DLL, allow the application to import definitions
  #define UWBGL_API __declspec(dllimport)
#else
  // We are compiling as a library that will be staticly linked into our project
  #define UWBGL_API
#endif



#ifndef VC_EXTRALEAN
  #define VC_EXTRALEAN          // Exclude rarely-used stuff from Windows headers
#endif
#ifndef WINVER                  // Allow features specific to Win95 and WinNT 4 or later.
  #define WINVER 0x0400         // Change to target later windows versions.
#endif
#ifndef _WIN32_WINNT            // Allow features specific to WinNT 4 or later.
  #define _WIN32_WINNT 0x0400   // Change to target later WinNT versions.
#endif						
#ifndef _WIN32_WINDOWS          // Allow features specific to Windows 98 or later.
  #define _WIN32_WINDOWS 0x0410 // Change to target Windows Me or later.
#endif
#ifndef _WIN32_IE               // Allow features specific to IE 4.0 or later.
  #define _WIN32_IE 0x0400      // Change to target IE 5.0 or later.
#endif

#define _ATL_CSTRING_EXPLICIT_CONSTRUCTORS // some CString constructors will be explicit
#include "Windows.h"

#include "math3d++/math3d++.h"      // math library
using namespace math3d;

// Level of detail enum for supporting multiple detail levels on graphics objects
enum eLevelOfDetail { lodLow, lodMedium, lodHigh };
