// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////

#pragma once
#include <vector>

/// Very thin wrapper around the STL vector class to make dynamic arrays easier to
/// use and we can swap out the vector for a different implementation if
/// we ever want to in the future. The wrapped vector is made public in case there is
/// any need to access any additional features offered by the vector itslef
///
/// UWB_Array class should be used to hold basic data types (int, char, float, ...)
/// or class instances. If you want a container that holds pointers, use the UWB_PointerArray
template <class T> class UWB_Array
{
public:
  UWB_Array(){}

  UWB_Array(int capacity):m_array(capacity){}

  UWB_Array( const UWB_Array<T>& other ){m_array = other.m_array;}

  virtual ~UWB_Array(){}

  virtual UWB_Array<T>& operator=( const UWB_Array<T>& other )
  {
    m_array = other.m_array;
    return *this;
  }
  
  int Count() const {return (int)m_array.size();}
  //int Capacity() const{return (int)m_array.

  T& operator[]( int index ){return m_array[index];}
  const T& operator[]( int index) const{return m_array[index];}

  void Append( const T& item ){m_array.push_back(item);}

  void Insert( const T& item, int index )
  {
    if( index < 0 )
      index = 0;
    if( index >= Count() )
    {
      Append(item);
    }
    else
    {
      std::vector<T>::iterator it;
      it = m_array.begin() + index;
      m_array.insert(it, item);
    }
  }

  virtual void DeleteItem( int index)
  {
    std::vector<T>::iterator it;
    it = m_array.begin() + index;
    m_array.erase(it);
  }

  std::vector<T> m_array;
};


// UWB_PointerArray is used to contain pointers to objects (basic data types or classes)
// The difference between this class and the UWB_Array is that this class will delete the
// objects in the destructor if the user specifies it.
template <class T> class UWB_PointerArray : public UWB_Array<T>
{
public:
  UWB_PointerArray(bool bDeleteInDestructor = true):m_bDeleteInDestructor(bDeleteInDestructor){}
  
  UWB_PointerArray(int capacity, bool bDeleteInDestructor = true)
  : UWB_Array<T>(capacity)
  , m_bDeleteInDestructor(bDeleteInDestructor)
  {
  }

  virtual ~UWB_PointerArray()
  {
    if(m_bDeleteInDestructor)
    {
      for(int i=0; i<Count(); i++)
      {
        T pItem = m_array[i];
        delete pItem;
        pItem = NULL;
      }
    }
  }

  virtual void DeleteItem(int index)
  {
    T pItem = m_array[index];
    __super::DeleteItem(index);
    delete pItem;
  }

  bool m_bDeleteInDestructor;
};
