#include "uwbgl_D3DCircleGeom2.h"
#include "uwbgl_D3DGraphicsSystem2.h"
#include <math.h>

UWBD3D_CircleGeometry::UWBD3D_CircleGeometry()
:m_center(0,0,0)
,m_radius(0)
,m_bFill(true)
{
}

void UWBD3D_CircleGeometry::SetCenter(float x, float y, float z)
{
  m_center = vec3(x,y,z);
}

void UWBD3D_CircleGeometry::SetCenter(vec3 center)
{
  m_center = center;
}

vec3 UWBD3D_CircleGeometry::GetCenter() const
{
  return m_center;
}

void UWBD3D_CircleGeometry::SetRadius(float radius)
{
  m_radius = radius;
}

float UWBD3D_CircleGeometry::GetRadius() const
{
  return m_radius;
}

void UWBD3D_CircleGeometry::SetFillColor(const UWB_Color& color)
{
  m_fill_color = color;
}

UWB_Color UWBD3D_CircleGeometry::GetFillColor() const
{
  return m_fill_color;
}

void UWBD3D_CircleGeometry::SetFillMode(bool on)
{
  m_bFill = on;
}

bool UWBD3D_CircleGeometry::IsFillModeOn() const
{
  return m_bFill;
}

void UWBD3D_CircleGeometry::Draw() const
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct DeviceVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    DWORD m_color;        // color at the vertex
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if (m_radius <= 0.0f || NULL == pDevice)
    return;

  // We are going to draw this circle with TriangleFan
  const int kNumPts = 100;
  DeviceVertexFormat v[kNumPts+2];
  
  float delta = (float)((2.0f * 3.14159265f) / kNumPts);
  D3DCOLOR vertex_color = m_fill_color.ARGB();

  v[0].m_point.x = m_center.x;
  v[0].m_point.y = m_center.y;
  v[0].m_point.z = 0;
  v[0].m_color = vertex_color;

  for (int i=0; i<(kNumPts); i++)
  {
    v[i+1].m_point.x = m_center.x + (m_radius * cosf(i*delta));
    v[i+1].m_point.y = m_center.y + (m_radius * sinf(i*delta));
    v[i+1].m_point.z = 0;
    v[i+1].m_color = v[0].m_color;
  }
  v[kNumPts+1] = v[1];
  
  pDevice->SetFVF(D3DFVF_XYZ | D3DFVF_DIFFUSE);
  if(m_bFill)
    pDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, kNumPts, (CONST void *)v, sizeof(DeviceVertexFormat));
  else
  {
    int num_edge_pts = kNumPts;
    DeviceVertexFormat* edge_start = &(v[1]);
    pDevice->DrawPrimitiveUP(D3DPT_LINESTRIP, num_edge_pts, (CONST void *)edge_start, sizeof(DeviceVertexFormat));
  }
}
