// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_D3DDefines.h"
#include "uwbgl_Color1.h"

/// Represents a 2-dimensional circle that can be drawn to a graphics window
/// with a solid fill color
class UWBD3D_CircleGeometry
{
public:
  UWBD3D_CircleGeometry();

  void SetCenter(float x, float y, float z = 0.0f);
  void SetCenter(vec3 center);
  vec3 GetCenter() const;

  void SetRadius(float radius);
  float GetRadius() const;

  void SetFillColor(const UWB_Color& color);
  UWB_Color GetFillColor() const;

  void SetFillMode(bool on);
  bool IsFillModeOn() const;

  /// Draw this circle to the active graphics device (graphics window)
  /// if the circle is valid ( radius > 0 )
  void Draw() const;

protected:
  vec3 m_center;    // let's keep all points in 3D space with z=0
  float m_radius;   // Radius of the circle
  UWB_Color m_fill_color;
  bool m_bFill;     // if true, the circle is drawn as a filled disk
                    // if false, only the edge is drawn
};
