// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_D3DDefines.h"

/// Direct3D API specific calls are abstracted off to this class.
/// This is a "singleton" class. Only a single instance of this class ever exists.
/// Notice that the constructor is private.
class UWBD3D_GraphicsSystem
{
public:
  ///
  /// static method for getting a reference to the "singleton" system
  static UWBD3D_GraphicsSystem& GetSystem();
private:
  ///
  /// the one and only instance of the system
  static UWBD3D_GraphicsSystem m_TheAPI;

public:
  /// Create a connection to Direct3D for a given window
  /// \param hWindow Handle to the window that we want to use Direct3D in
  /// \return true on success
  bool CreateGraphicsContext(HWND hMainWindow);

  /// Force the graphics system to release all of it's current connections
  /// to Direct3D. This is typically called when the window that is displaying
  /// Direct3D graphics is being destroyed.
  /// The connections are released, but new connections could be made by calling
  /// CreateGraphicsContext again
  void ShutDown();

  // returns true if GraphicsSystem has already been initialized
  // this is handy funciton to call to determine if system 
  // initialization has happen yet.
  bool IsInitialized() const;

  /// This graphics system supports a single graphics device context. Call
  /// GetD3DDevice to retrieve a pointer to this device.
  /// \return pointer the the Direct3D device if one has been created (see CreateGraphicsContext)
  LPDIRECT3DDEVICE9 GetD3DDevice();

  /// Create a "swap chain" for a specific window. This is the technique we use to
  /// draw Direct3D graphics into multiple windows
  LPDIRECT3DSWAPCHAIN9 CreateSwapChain(HWND hWindow);
  virtual ~UWBD3D_GraphicsSystem();

private:
  /// The constructor is made private because we don't want people to be able to
  /// create an instance of this class.
  UWBD3D_GraphicsSystem();

  ///
  /// Abstract connection to the Direct3D graphics system.
  LPDIRECT3D9         m_pD3D;

  /// Direct3D connection specific to a window (device)
  /// One per application because we will use swap chains to support multiple windows.
  LPDIRECT3DDEVICE9   m_pD3DDevice;

  ///
  ///counter for the current number of swap chains created
  int                 m_SwapChainCounter; 
};
