#include "uwbgl_D3DRectangleGeom2.h"
#include "uwbgl_D3DGraphicsSystem2.h"

UWBD3D_RectangleGeometry::UWBD3D_RectangleGeometry()
: m_center(0,0,0)
, m_width(0)
, m_height(0)
{
}

void UWBD3D_RectangleGeometry::SetCenter( vec3 center)
{
  m_center = center;
}

void UWBD3D_RectangleGeometry::SetSize(float width, float height)
{
  m_width = width;
  m_height = height;
}

void UWBD3D_RectangleGeometry::Draw() const
{
  //Direct3D likes to recieve arrays of structs with vertex information. For simple, unlit
  //drawing; a vertex and color at the vertex will suffice.
  struct DeviceVertexFormat
  {
    D3DXVECTOR3 m_point;  // vertex location
    DWORD m_color;        // color at the vertex
  };

  LPDIRECT3DDEVICE9 pDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(pDevice)
  {
    //Draw the rectangle
    DeviceVertexFormat v[4];

    //all of the vertices have the same color
    for(int i=0; i<4; i++)
    {
      ZeroMemory(&(v[i]), sizeof(DeviceVertexFormat));
      v[i].m_color = D3DCOLOR_COLORVALUE(0.4f, 0.4f, 0.4f, 1);
    }

    //set the corner point locations based on the rectangle geometry
    v[0].m_point = D3DXVECTOR3(m_center.x - m_width/2.0f, m_center.y + m_height/2.0f, 0);
    v[1].m_point = D3DXVECTOR3(m_center.x - m_width/2.0f, m_center.y - m_height/2.0f, 0);
    v[2].m_point = D3DXVECTOR3(m_center.x + m_width/2.0f, m_center.y - m_height/2.0f, 0);
    v[3].m_point = D3DXVECTOR3(m_center.x + m_width/2.0f, m_center.y + m_height/2.0f, 0);
    pDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, 2, (CONST void *)v, sizeof(DeviceVertexFormat));
  }
}