#include "uwbgl_D3DWindowHandler3.h"
#include "uwbgl_D3DGraphicsSystem2.h"


UWBD3D_WindowHandler::UWBD3D_WindowHandler()
: m_pD3DDevice(NULL)
, m_pSwapChain(NULL)
, m_pDrawSurface(NULL)
{
}

UWBD3D_WindowHandler::~UWBD3D_WindowHandler()
{
  // release pointer to D3D device
  ShutDownHandler();
}

bool UWBD3D_WindowHandler::IsGraphicsDeviceValid()
{
  return (m_pD3DDevice != NULL && m_pSwapChain != NULL);
}

bool UWBD3D_WindowHandler::InitializeHandler(HWND hAttachedWindow)
{
  //Release connections if any already exist
  ShutDownHandler();

  // call parent class function to create graphics connection 
  if( !__super::InitializeHandler(hAttachedWindow) )
    return false;

  // Get the device from the graphics system.  Will be NULL if not yet set.
  m_pD3DDevice = UWBD3D_GraphicsSystem::GetSystem().GetD3DDevice();
  if(NULL == m_pD3DDevice)
    return false;

  m_pD3DDevice->AddRef();
  m_pSwapChain = UWBD3D_GraphicsSystem::GetSystem().CreateSwapChain(hAttachedWindow);

  return (NULL != m_pD3DDevice && NULL != m_pSwapChain);
}

void UWBD3D_WindowHandler::ShutDownHandler()
{
  if(m_pSwapChain)
  {
    m_pSwapChain->Release();
    m_pSwapChain = NULL;
  }
  if(m_pD3DDevice)
  {
    m_pD3DDevice->Release();
    m_pD3DDevice = NULL;
  }
}

void UWBD3D_WindowHandler::BeginDraw()
{
  if(NULL == m_pSwapChain || NULL == m_pD3DDevice)
    return;
  if (m_pDrawSurface )
    m_pDrawSurface->Release();
  m_pDrawSurface = NULL;
  m_pSwapChain->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &m_pDrawSurface );
  m_pD3DDevice->SetRenderTarget( 0, m_pDrawSurface );

  m_pD3DDevice->BeginScene();
}

void UWBD3D_WindowHandler::EndDrawAndShow()
{
  if(m_pD3DDevice)
    m_pD3DDevice->EndScene();

  //show the draw buffer
  if(m_pSwapChain)
    m_pSwapChain->Present(NULL, NULL, m_hAttachedWindow, NULL, NULL);

  if (m_pDrawSurface)
  {
    m_pDrawSurface->Release();
    m_pDrawSurface = NULL;
  }
}