// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_IWindowHandler.h"
#include "uwbgl_Common.h"

///
/// Common base class for graphics window handlers.
class UWB_WindowHandler : public UWB_IWindowHandler
{
public:
  UWB_WindowHandler();
  virtual ~UWB_WindowHandler();

  /// Subclasses should always call this when overriding this function. Checks that
  /// hParentWindow is an actual window and saves it to m_hAttachedWindow
  virtual bool InitializeHandler(HWND hAttachedWindow);

  /// Default implementation does nothing
  virtual void ShutDownHandler(){}

  /// Default mouse event handlers do nothing
  virtual void OnMouseButton(bool down, unsigned int nFlags, int hwX, int hwY){}
  virtual void OnMouseMove(unsigned int nFlags, int hwX, int hwY){}


  /// Transform point from hardware coordinates to device coordinates
  /// \param hardware_x, hardware_y [in]: hardware (or MFC window client) coordinate to transform
  /// \param device_x, device_y [out]: resulting device coordinate
  /// return true if successful
  bool HardwareToDevice(int hardwareX, int hardwareY, int& deviceX, int& deviceY) const;

  /// return aspect ratio of viewport (aspect ratio = width / height)
  float GetAspectRatio() const;
protected:
  ///
  /// Handle to window that this is connected to
  HWND m_hAttachedWindow;
};
