#include "uwbgl_MFCSliderCtrlWithEcho1.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

// This macro adds IsKindOf function to this class to provide for
// runtime type information
IMPLEMENT_DYNAMIC(UWBMFC_SliderCtrlWithEcho, CSliderCtrl)

UWBMFC_SliderCtrlWithEcho::UWBMFC_SliderCtrlWithEcho()
: m_minimum_value(0.0f)
, m_value_range(1.0f)
{
}

UWBMFC_SliderCtrlWithEcho::~UWBMFC_SliderCtrlWithEcho(){}


void UWBMFC_SliderCtrlWithEcho::Initialize(float min, float max, float init)
{
  // check for valid input and that this control exists as a window
  if( !IsWindow(m_hWnd) || max <= min || init<min || init>max)
    return;

  // Create the buddy text control if it has not already been created
  if( !IsWindow(m_MessageWnd.m_hWnd) )
  {
    // Get the parent window for this slider control
    CWnd* pParent = GetParent();
    if( pParent )
    {
      CRect controlRect;
      GetWindowRect(&controlRect);
      CRect msgRect(controlRect);

      if( !IsOrientedVertical() )
      {
        msgRect.left = msgRect.right-50;
        controlRect.right = controlRect.right-50;
      }
      else
      {
        msgRect.left = controlRect.left+controlRect.Width()/2-50/2;
        msgRect.right = msgRect.left+50;
        msgRect.top = controlRect.top;
        msgRect.bottom = msgRect.top+20;
        controlRect.top = controlRect.top+20;
      }

      pParent->ScreenToClient(&msgRect);
      pParent->ScreenToClient(&controlRect);

      m_MessageWnd.Create( L"XXXX",SS_CENTERIMAGE|WS_CHILD|WS_VISIBLE|SS_SUNKEN,msgRect,pParent);
      MoveWindow(&controlRect);
    
      // Make the control's font match the font used on the parent window
      m_MessageWnd.SetFont( pParent->GetFont() );
    }
  }

  m_minimum_value = min;
  m_value_range = max - min;
  SetRange(0, SLIDER_RESOLUTION, TRUE);
  SetSliderValue(init);
  UpdateSliderEcho();
}


BEGIN_MESSAGE_MAP(UWBMFC_SliderCtrlWithEcho, CSliderCtrl)
  ON_WM_HSCROLL_REFLECT()
  ON_WM_VSCROLL_REFLECT()
  ON_WM_SHOWWINDOW()
END_MESSAGE_MAP()

void UWBMFC_SliderCtrlWithEcho::HScroll(UINT nSBCode, UINT nPos)
{
  int slider_position = GetPos();
  float userValue = ConvertSliderPositionToUserValue( slider_position );
  // update the text in our static control
  UpdateSliderEcho();
  // allow subclasses to respond to this event
  OnScroll(userValue);
}
void UWBMFC_SliderCtrlWithEcho::VScroll(UINT nSBCode, UINT nPos)
{
  int slider_position = GetPos();
  float userValue = ConvertSliderPositionToUserValue( slider_position );
  // update the text in our static control
  UpdateSliderEcho();
  // allow subclasses to respond to this event
  OnScroll(userValue);
}
void UWBMFC_SliderCtrlWithEcho::OnShowWindow(BOOL bShow, UINT nStatus)
{
  CSliderCtrl::OnShowWindow(bShow, nStatus);
  // update the static buddy text control visibility state to
  // match this slider control
  if( IsWindow(m_MessageWnd.m_hWnd) )
  {
    if(bShow)
      m_MessageWnd.ShowWindow(SW_SHOW);
    else
      m_MessageWnd.ShowWindow(SW_HIDE);
  }
}


bool UWBMFC_SliderCtrlWithEcho::SetSliderValue(float userValue)
{
  if(userValue < m_minimum_value || userValue > (m_minimum_value+m_value_range) )
    return false;
  int controlPosition = ConvertUserValueToSliderPosition(userValue);
  SetPos(controlPosition);
  UpdateSliderEcho();
  return true;
}

float UWBMFC_SliderCtrlWithEcho::GetSliderValue()
{
  int slider_position = GetPos();
  if( IsOrientedVertical())
    slider_position = SLIDER_RESOLUTION - slider_position;

  return ConvertSliderPositionToUserValue( slider_position );
}

int UWBMFC_SliderCtrlWithEcho::ConvertUserValueToSliderPosition(float userValue) const
{
  int slider_position = int(((userValue - m_minimum_value) * SLIDER_RESOLUTION / m_value_range)+0.5f);
  if(IsOrientedVertical())
    slider_position = SLIDER_RESOLUTION - slider_position;
  return slider_position;
}

float UWBMFC_SliderCtrlWithEcho::ConvertSliderPositionToUserValue( int position ) const
{
  return m_minimum_value + (float(position) / float(SLIDER_RESOLUTION) * m_value_range );
}

void UWBMFC_SliderCtrlWithEcho::UpdateSliderEcho()
{
  if(IsWindow(m_MessageWnd.m_hWnd))
  {
    CString msg;
    msg.Format(L" %.2f",GetSliderValue());
    m_MessageWnd.SetWindowText(msg);
  }
}

void UWBMFC_SliderCtrlWithEcho::OnScroll(float new_value)
{
}

bool UWBMFC_SliderCtrlWithEcho::IsOrientedVertical() const
{
  bool bVert = false;
  if( IsWindow(m_hWnd ) )
  {
    DWORD style = GetStyle();
    bVert = ( TBS_VERT == (style & TBS_VERT) );
  }
  return bVert;
}